import pytest
from pymongo_migrate.migrations import Migration, MigrationState
from pymongo_migrate.mongo_migrate import dt


@pytest.fixture
def migration():
    return Migration(name="2019_test")


def test_get_state_when_migration_not_applied(mongo_migrate, db, migration):
    assert mongo_migrate.get_state(migration) == MigrationState(
        name=migration.name, applied=None
    )


def test_get_state_when_migration_not_applied_but_in_db(
    mongo_migrate, db_collection, migration
):
    db_collection.insert_one({"name": migration.name, "applied": None})
    assert mongo_migrate.get_state(migration) == MigrationState(
        name=migration.name, applied=None
    )


def test_get_state_when_migration_applied(mongo_migrate, db_collection, migration):
    now = dt().replace(microsecond=0)  # reduced time resolution
    db_collection.insert_one({"name": migration.name, "applied": now})
    assert mongo_migrate.get_state(migration) == MigrationState(
        name=migration.name, applied=now
    )


def test_set_state(mongo_migrate, migration, get_db_migrations):
    now = dt().replace(microsecond=0)  # reduced time resolution
    mongo_migrate.set_state(MigrationState(name=migration.name, applied=now))
    all_migrations = get_db_migrations()
    assert all_migrations == [{"name": migration.name, "applied": now}]



import importlib.util
from pathlib import Path
from typing import Generator, cast

from pymongo_migrate.migrations import MigrationModuleType, MigrationModuleWrapper


def load_module_migrations(
    path: Path, namespace=f"{__name__}._migrations"
) -> Generator[MigrationModuleWrapper, None, None]:
    for module_file in path.glob("*.py"):
        if module_file.name.startswith("__"):
            continue
        migration_name = module_file.stem
        spec = importlib.util.spec_from_file_location(
            f"{namespace}.{migration_name}", str(module_file)
        )
        assert spec
        migration_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(migration_module)  # type: ignore
        yield MigrationModuleWrapper(
            name=migration_name, module=cast(MigrationModuleType, migration_module)
        )



import freezegun
import pymongo
import pytest
from pymongo_migrate.mongo_migrate import MongoMigrate


@freezegun.freeze_time("2019-02-25 01:13:56")
def test_generate(mongo_migrate, tmp_path):
    tmp_migrations_path = tmp_path / "test_generate_migrations"
    mongo_migrate.migrations_dir = str(tmp_migrations_path)
    filename = mongo_migrate.generate().name
    assert filename == "20190225011356.py"
    files = {f.name: f for f in tmp_migrations_path.iterdir()}
    assert set(files) == {filename}
    with files[filename].open() as f:
        content = f.read()
        assert (
            content
            == '''\
"""
Migration description here!
"""
import pymongo

name = '20190225011356'
dependencies = ['20181123000000_gt_500']


def upgrade(db: "pymongo.database.Database"):
    pass


def downgrade(db: "pymongo.database.Database"):
    pass
'''
        )


@pytest.fixture
def empty_migrations_dir(tmp_path):
    yield tmp_path / "migrations"


@pytest.fixture
def mongo_migrate_with_empty_dir(db_uri, db_name, db, empty_migrations_dir):
    mm = MongoMigrate(
        pymongo.MongoClient(db_uri), db_name, migrations_dir=str(empty_migrations_dir)
    )
    yield mm
    mm.client.close()


@freezegun.freeze_time("2019-02-03 04:05:06")
def test_generate_initial_migration(mongo_migrate_with_empty_dir, empty_migrations_dir):
    mongo_migrate = mongo_migrate_with_empty_dir
    filename = mongo_migrate.generate().name
    assert filename == "20190203040506.py"
    files = {f.name: f for f in empty_migrations_dir.iterdir()}
    assert set(files) == {filename}
    with files[filename].open() as f:
        content = f.read()
        assert (
            content
            == '''\
"""
Migration description here!
"""
import pymongo

name = '20190203040506'
dependencies = []


def upgrade(db: "pymongo.database.Database"):
    pass


def downgrade(db: "pymongo.database.Database"):
    pass
'''
        )


def test_generate_should_fail_when_name_collides(mongo_migrate):
    with pytest.raises(FileExistsError):
        mongo_migrate.generate("20181123000000_gt_500")



import datetime
import re
from pathlib import Path
from typing import List, Optional

MIGRATION_MODULE_TMPL = '''\
"""
{description}
"""
import pymongo

name = {name!r}
dependencies = {dependencies!r}


def upgrade(db: "pymongo.database.Database"):
    pass


def downgrade(db: "pymongo.database.Database"):
    pass
'''
MAX_NAME_LEN = 60


def slugify(text: str) -> str:
    text = re.sub(r"[\W_]+", "_", text)
    text = text.encode("ascii", errors="ignore").decode().lower()
    text = text.strip("_")
    return text


def generate_migration_module(
    fp,
    name: str,
    description: str = "Migration description here!",
    dependencies: Optional[List[str]] = None,
):
    dependencies = dependencies or []

    content = MIGRATION_MODULE_TMPL.format(
        name=name, description=description, dependencies=dependencies
    )
    fp.write(content)


def generate_migration_module_in_dir(
    migration_dir: Path, name: str = "", *args, **kwargs
) -> Path:
    now = datetime.datetime.utcnow()
    if not name:
        name = f"{now:%Y%m%d%H%M%S}"
        description = kwargs.get("description")
        if description:
            name = f"{name}_{slugify(description)}"
        name = name[:MAX_NAME_LEN]
    file_path = migration_dir / f"{name}.py"
    if file_path.exists():
        raise FileExistsError(f"{file_path} already exists")

    with file_path.open("w") as f:
        generate_migration_module(f, name=name, *args, **kwargs)  # type: ignore
    return file_path



"""Draw migrations graph"""

from io import StringIO
from typing import TextIO

from pymongo_migrate.migrations import MigrationsGraph


def dump(graph: MigrationsGraph, output_file: TextIO):
    output_file.write("// Migrations\n")
    output_file.write("digraph {\n")
    for migration in graph.migrations.values():
        for dependency_name in migration.dependencies:
            dependency = graph.migrations[dependency_name]
            output_file.write(f"\t{dependency.name!r} -> {migration.name!r}\n")
    output_file.write("}\n")


def dumps(graph: MigrationsGraph) -> str:
    with StringIO() as f:
        dump(graph, f)
        return f.getvalue()



import datetime
import logging
import time
from dataclasses import asdict, dataclass
from pathlib import Path
from typing import Iterator, Optional

import pymongo
from bson import CodecOptions

from pymongo_migrate.generate import generate_migration_module_in_dir
from pymongo_migrate.loader import load_module_migrations
from pymongo_migrate.migrations import Migration, MigrationsGraph, MigrationState

LOGGER = logging.getLogger(__name__)


def dt(*args) -> datetime.datetime:
    """Create timezone-aware UTC datetime."""
    if args:
        return datetime.datetime(*args, tzinfo=datetime.timezone.utc)  # type: ignore
    else:
        return datetime.datetime.now(datetime.timezone.utc)


def _serialize(obj):
    return asdict(obj)


def _deserialize(data, cls):
    data = {**data}
    del data["_id"]
    return cls(**data)


class _MeasureTime:
    """
    Class to measure the time of execution of code block.
    usage:

     with MeasureTime() as mt:
         # code block ...
         print(f"Execution time: {mt.elapsed}s")
    """

    def __init__(self):
        self.start = None
        self.stop = None

    @staticmethod
    def time() -> float:
        return time.time()

    @property
    def elapsed(self) -> Optional[float]:
        if self.start is None:
            return None
        return (self.stop or self.time()) - self.start

    def __enter__(self):
        self.start = self.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop = self.time()


@dataclass
class MongoMigrate:
    client: pymongo.MongoClient
    database: str
    migrations_dir: str = "./pymongo_migrations"
    migrations_collection: str = "pymongo_migrate"
    logger: logging.Logger = LOGGER

    def __post_init__(self):
        self.graph = MigrationsGraph()
        for migration in load_module_migrations(self.migrations_path):
            self.graph.add_migration(migration)
        self.graph.verify()

    @property
    def migrations_path(self):
        return Path(self.migrations_dir)

    @property
    def db(self):
        return self.client.get_database(self.database)

    @property
    def db_collection(self):
        return self.client.get_database(self.database)[
            self.migrations_collection
        ].with_options(
            codec_options=CodecOptions(tz_aware=True, tzinfo=datetime.timezone.utc)
        )

    def get_migrations(self) -> Iterator[Migration]:
        yield from self.graph

    def get_state(self, migration: Migration) -> MigrationState:
        data = self.db_collection.find_one({"name": migration.name})
        if data:
            return _deserialize(data, MigrationState)
        return MigrationState(name=migration.name)

    def set_state(self, status: MigrationState):
        self.db_collection.replace_one(
            {"name": status.name}, _serialize(status), upsert=True
        )

    def _check_for_migration(
        self, migration_name: Optional[str]
    ) -> Optional[Migration]:
        if migration_name is None:
            return None
        migration = self.graph.migrations.get(migration_name)
        if migration is None:
            raise ValueError(f"No such migration: {migration_name}")
        return migration

    def migrate(self, migration_name: Optional[str] = None, fake: bool = False):
        """
        Automatically detects if upgrades or downgrades should be applied to
        reach target migration state.

        :param migration_name:
            target migration
            None if all upgrades should be applied
        :param fake:
            If True, only migration state in database will be modified and
            no actual migration will be run.
        """
        if migration_name is None:
            self.logger.debug("Migration target not specified, assuming upgrade")
            self.upgrade(fake=fake)
            return
        migration = self._check_for_migration(migration_name)
        assert migration, "No matching migration, something went wrong"
        migration_state = self.get_state(migration)
        if migration_state.applied:
            self.logger.debug("Migration target already applied, assuming downgrade")
            self.downgrade(migration_name, fake)
        else:
            self.logger.debug("Migration target not applied, assuming upgrade")
            self.upgrade(migration_name, fake)

    def upgrade(self, migration_name: Optional[str] = None, fake: bool = False):
        """
        Apply upgrade migrations.

        :param migration_name:
            name of migration up to which (including) upgrades should be executed
            None if all migrations should be run
        :param fake:
            If True, only migration state in database will be modified and
            no actual migration will be run.
        """
        self._check_for_migration(migration_name)
        for migration in self.graph:
            migration_state = self.get_state(migration)
            if migration_state.applied:
                self.logger.debug(
                    "Migration %r already applied, skipping", migration.name
                )
                continue
            if fake:
                self.logger.info("Fake running upgrade migration %r", migration.name)
            else:
                self.logger.info("Running upgrade migration %r", migration.name)
                with _MeasureTime() as mt:
                    migration.upgrade(self.db)
                    self.logger.info(
                        "Execution time of %r: %s seconds", migration.name, mt.elapsed
                    )
            migration_state.applied = dt()
            self.set_state(migration_state)
            if migration.name == migration_name:
                break

    def downgrade(self, migration_name: Optional[str] = None, fake: bool = False):
        """
        Reverse migrations.

        :param migration_name:
            name of migration down to which (excluding) downgrades should be executed
            None if all migrations should be run
        :param fake:
            If True, only migration state in database will be modified and
            no actual migration will be run.
        """
        self._check_for_migration(migration_name)
        for migration in reversed(list(self.get_migrations())):
            if migration.name == migration_name:
                break
            migration_state = self.get_state(migration)
            if not migration_state.applied:
                self.logger.debug(
                    "Migration %r not yet applied, skipping", migration.name
                )
                continue
            if fake:
                self.logger.info("Fake running downgrade migration %r", migration.name)
            else:
                self.logger.info("Running downgrade migration %r", migration.name)
                with _MeasureTime() as mt:
                    migration.downgrade(self.db)
                    self.logger.info(
                        "Execution time of %r: %s seconds", migration.name, mt.elapsed
                    )
            migration_state.applied = None
            self.set_state(migration_state)

    def generate(self, name: str = "", **kwargs) -> Path:
        last_migration_name = None
        for migration in self.get_migrations():
            last_migration_name = migration.name
        self.migrations_path.mkdir(exist_ok=True)
        dependencies = [last_migration_name] if last_migration_name else []
        return generate_migration_module_in_dir(
            self.migrations_path,
            name=name,
            dependencies=dependencies,
            **{k: v for k, v in kwargs.items() if v is not None},
        )



import datetime
from collections import defaultdict
from dataclasses import dataclass, field
from types import ModuleType
from typing import Dict, List, Optional, Set

from pymongo.database import Database


@dataclass
class Migration:
    name: str
    dependencies: List[str] = field(default_factory=list)

    @property
    def initial(self):
        return not self.dependencies

    def upgrade(self, db: Database):
        raise NotImplementedError()

    def downgrade(self, db: Database):
        raise NotImplementedError()


class MigrationModuleType(ModuleType):
    """Migration Module Type stub"""

    name: str
    dependencies: List[str]

    def upgrade(self, db: Database):
        raise NotImplementedError()

    def downgrade(self, db: Database):
        raise NotImplementedError()


class MigrationModuleWrapper(Migration):
    """Use python module as a migration"""

    def __init__(self, name: str, module: MigrationModuleType):
        self.name = name
        self.module = module
        assert self.name == getattr(self.module, "name", name)
        super().__init__(name=name, dependencies=self.module.dependencies)

    @property
    def description(self):
        return self.module.__doc__

    def upgrade(self, db: Database):
        self.module.upgrade(db)

    def downgrade(self, db: Database):
        self.module.downgrade(db)


class MigrationsGraph:
    def __init__(self):
        self.migrations: Dict[str, Migration] = {}
        self.required_by: Dict[str, Set[str]] = defaultdict(set)

    def add_migration(self, migration: Migration):
        self.migrations[migration.name] = migration
        for required_migration_name in migration.dependencies:
            self.required_by[required_migration_name].add(migration.name)

    def get_initial(self):
        initial_migrations = [m for m in self.migrations.values() if m.initial]
        if len(initial_migrations) != 1:
            raise ValueError("There must be single initial migration")
        return initial_migrations[0]

    def verify(self):
        ordered_migration_names = [migration.name for migration in self]
        all_migrations = set(self.migrations.keys())
        unused_migrations = all_migrations - set(ordered_migration_names)
        if unused_migrations:
            raise ValueError("Migration graph is disconnected")

    def __iter__(self):
        """
        Iterate over migrations starting with initial one
        """
        if not self.migrations:
            return
        initial_migration = self.get_initial()
        yield from self._get_next(initial_migration)

    def _get_next(self, migration):
        yield migration
        for next_migration_name in sorted(self.required_by.get(migration.name, [])):
            next_migration = self.migrations[next_migration_name]
            yield from self._get_next(next_migration)


@dataclass
class MigrationState:
    name: str
    applied: Optional[datetime.datetime] = None



from collections import defaultdict
from unittest.mock import Mock, patch

import freezegun
from pymongo_migrate.mongo_migrate import dt


def test_get_migrations(mongo_migrate):
    assert [migration.name for migration in mongo_migrate.get_migrations()] == [
        "20150612230153",
        "20181123000000_gt_500",
    ]


@freezegun.freeze_time("2019-02-03 04:05:06")
def test_upgrade(mongo_migrate, db, get_db_migrations):
    mongo_migrate.upgrade()

    db_migrations = get_db_migrations()
    assert db_migrations == [
        {"applied": dt(2019, 2, 3, 4, 5, 6), "name": "20150612230153"},
        {"applied": dt(2019, 2, 3, 4, 5, 6), "name": "20181123000000_gt_500"},
    ]

    assert db.numbers_collection.count_documents({}) == 499


@freezegun.freeze_time("2019-02-03 04:05:06")
def test_upgrade_fake(mongo_migrate, db, get_db_migrations):
    mongo_migrate.upgrade(fake=True)

    db_migrations = get_db_migrations()
    assert db_migrations == [
        {"applied": dt(2019, 2, 3, 4, 5, 6), "name": "20150612230153"},
        {"applied": dt(2019, 2, 3, 4, 5, 6), "name": "20181123000000_gt_500"},
    ]

    assert db.numbers_collection.count_documents({}) == 0


@freezegun.freeze_time("2019-02-03 04:05:06")
def test_upgrade_skip_applied(mongo_migrate, db, db_collection, get_db_migrations):
    db_collection.insert_one(
        {"applied": dt(2017, 10, 10, 10, 10, 10), "name": "20150612230153"}
    )

    migrations = {m.name: m for m in mongo_migrate.get_migrations()}
    upgrade_mocks = defaultdict(Mock)

    def patch_upgrade(name):
        return patch.object(migrations[name], "upgrade", upgrade_mocks[name])

    with patch_upgrade("20150612230153"), patch_upgrade("20181123000000_gt_500"):
        mongo_migrate.upgrade()
    upgrade_mocks["20150612230153"].assert_not_called()
    upgrade_mocks["20181123000000_gt_500"].assert_called()

    assert get_db_migrations() == [
        {"applied": dt(2017, 10, 10, 10, 10, 10), "name": "20150612230153"},
        {"applied": dt(2019, 2, 3, 4, 5, 6), "name": "20181123000000_gt_500"},
    ]


def test_downgrade(mongo_migrate, db, get_db_migrations):
    mongo_migrate.downgrade(None)
    db_migrations = get_db_migrations()
    assert db_migrations == []
    assert db.list_collection_names() == []


def test_downgrade_fake(mongo_migrate, db, get_db_migrations):
    mongo_migrate.upgrade()
    mongo_migrate.downgrade(fake=True)

    db_migrations = get_db_migrations()
    assert db_migrations == [
        {"applied": None, "name": "20150612230153"},
        {"applied": None, "name": "20181123000000_gt_500"},
    ]

    assert db.numbers_collection.count_documents({}) == 499


def test_upgrade_n_downgrade(mongo_migrate, db, get_db_migrations):
    mongo_migrate.upgrade()
    mongo_migrate.downgrade(None)
    db_migrations = get_db_migrations()
    assert db_migrations == [
        {"applied": None, "name": "20150612230153"},
        {"applied": None, "name": "20181123000000_gt_500"},
    ]
    assert db.list_collection_names() == ["pymongo_migrate"]


def test_migrate(mongo_migrate):
    """migrate without args should work as upgrade"""
    with patch.object(mongo_migrate, "upgrade") as upgrade_mock:
        mongo_migrate.migrate()
    upgrade_mock.assert_called()



import freezegun
import pytest
from click.testing import CliRunner
from pymongo_migrate.cli import cli


@pytest.fixture
def cli_runner():
    return CliRunner()


@pytest.fixture()
def invoker(cli_runner):
    def invoke_wrap(*args, **kwargs):
        return cli_runner.invoke(cli, *args, **kwargs)

    return invoke_wrap


def test_show(invoker, db, db_uri, migrations_dir):
    result = invoker(
        ["show", "-u", db_uri, "-m", migrations_dir], catch_exceptions=False
    )

    assert (
        result.stdout
        == """\
Migration name       	Applied timestamp
20150612230153       	Not applied
20181123000000_gt_500	Not applied
"""
    )


@freezegun.freeze_time("2019-02-03 04:05:06")
def test_generate(invoker, db, db_uri, tmp_path):
    migrations_path = tmp_path / "migrations"
    result = invoker(
        ["generate", "-u", db_uri, "-m", str(migrations_path)], catch_exceptions=False
    )
    filename = "20190203040506.py"
    assert filename in result.stdout
    files = {f.name: f for f in migrations_path.iterdir()}
    assert set(files) == {filename}
    with files[filename].open() as f:
        content = f.read()
        assert (
            content
            == '''\
"""
Migration description here!
"""
import pymongo

name = '20190203040506'
dependencies = []


def upgrade(db: "pymongo.database.Database"):
    pass


def downgrade(db: "pymongo.database.Database"):
    pass
'''
        )


def test_migrate(invoker, db, db_uri, migrations_dir):
    result = invoker(
        ["migrate", "-u", db_uri, "-m", migrations_dir], catch_exceptions=False
    )

    assert "Running upgrade migration '20150612230153'" in result.stdout


def test_migrate_fake(invoker, db, db_uri, migrations_dir):
    result = invoker(
        ["migrate", "-u", db_uri, "-m", migrations_dir, "--fake"],
        catch_exceptions=False,
    )

    assert "Fake running upgrade migration '20150612230153'" in result.stdout


def test_migrate_verbose(invoker, db, db_uri, migrations_dir):
    result = invoker(
        ["migrate", "-u", db_uri, "-m", migrations_dir, "-vv"], catch_exceptions=False
    )

    assert "Command update#" in result.stdout



import pytest
from pymongo_migrate.generate import slugify


@pytest.mark.parametrize(
    "test_input,expected",
    [
        ("My comment", "my_comment"),
        ("counting: 1, 2, 3", "counting_1_2_3"),
        ("wat?!", "wat"),
        ("mały żółwik", "may_wik"),
    ],
)
def test_slugify(test_input, expected):
    assert slugify(test_input) == expected



import logging
import os
from functools import wraps
from pprint import pformat
from typing import Optional

import click
import pymongo.monitoring

from pymongo_migrate.graph_draw import dump
from pymongo_migrate.mongo_migrate import MongoMigrate


@click.group()
def cli():
    pass


class CommandLogger(pymongo.monitoring.CommandListener):
    def __init__(self, verbose: int = 0):
        self.verbose = verbose

    def echo(self, *args, min_verbosity=1):
        if self.verbose >= min_verbosity:
            click.echo(*args)

    def started(self, event):
        self.echo(f"Command {event.command_name}#{event.request_id} STARTED")
        if self.verbose >= 1:
            self.echo(pformat(event.command), min_verbosity=1)

    def succeeded(self, event):
        self.echo(
            f"Command {event.command_name}#{event.request_id} SUCCEEDED in {event.duration_micros}us"
        )

    def failed(self, event):
        self.echo(
            f"Command {event.command_name}#{event.request_id} FAILED in {event.duration_micros}us"
        )


def get_logger(verbose: int):
    logger = logging.getLogger(__name__)

    stream = click.get_text_stream("stdout")
    console_handler = logging.StreamHandler(stream=stream)
    log_formatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s]  %(message)s")
    console_handler.setFormatter(log_formatter)
    logger.addHandler(console_handler)

    if verbose > 0:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)
    return logger


def mongo_migrate_decor(f):
    @wraps(f)
    def wrap_with_client(
        uri, database, migrations, collection, verbose, *args, **kwargs
    ):
        mongo_migrate = MongoMigrate(
            client=pymongo.MongoClient(
                uri, event_listeners=[CommandLogger(verbose=verbose)]
            ),
            database=database,
            migrations_dir=migrations,
            migrations_collection=collection,
            logger=get_logger(verbose),
        )
        return f(mongo_migrate, *args, **kwargs)

    return wrap_with_client


def _decorate(f, *decorators):
    for decorator in reversed(decorators):
        f = decorator(f)
    return f


def mongo_migration_options(f):
    return _decorate(
        f,
        click.option(
            "-u",
            "--uri",
            default=None,
            envvar="PYMONGO_MIGRATE_URI",
            help="mongodb URI",
        ),
        click.option(
            "-d",
            "--database",
            default=None,
            envvar="PYMONGO_MIGRATE_DATABASE",
            help="mongodb database on which to apply migrations",
        ),
        click.option(
            "-m",
            "--migrations",
            default=MongoMigrate.migrations_dir,
            envvar="PYMONGO_MIGRATE_MIGRATIONS",
            help="migration script directory",
            show_default=True,
        ),
        click.option(
            "-c",
            "--collection",
            default=MongoMigrate.migrations_collection,
            envvar="PYMONGO_MIGRATE_COLLECTION",
            help="mongodb collection used for storing migration states",
            show_default=True,
        ),
        click.option("-v", "--verbose", count=True),
        mongo_migrate_decor,
    )


@cli.command(short_help="show migrations and their status")
@mongo_migration_options
def show(mongo_migrate):
    name_len_max = max(
        (len(migration.name) for migration in mongo_migrate.get_migrations()), default=0
    )

    migration_column_name = "Migration name".ljust(name_len_max)
    click.secho(f"{migration_column_name}\tApplied timestamp", fg="yellow")
    for migration in mongo_migrate.get_migrations():
        migration_state = mongo_migrate.get_state(migration)
        if migration_state.applied:
            applied_text = click.style(migration_state.applied.isoformat(), fg="green")
        else:
            applied_text = click.style("Not applied", fg="red")
        click.echo(f"{migration.name.ljust(name_len_max)}\t" + applied_text)


def migrate_cmd_options(f):
    return _decorate(
        f,
        mongo_migration_options,
        click.argument("migration", required=False),
        click.option("--fake", is_flag=True),
    )


@cli.command(
    short_help="automagically apply necessary upgrades or downgrades to reach target migration"
)
@migrate_cmd_options
def migrate(mongo_migrate, migration=None, fake=False):
    mongo_migrate.migrate(migration, fake=fake)


@cli.command(short_help="apply necessary upgrades to reach target migration")
@migrate_cmd_options
def upgrade(mongo_migrate, migration=None, fake=False):
    mongo_migrate.upgrade(migration, fake=fake)


@cli.command(short_help="apply necessary downgrades to reach target migration")
@migrate_cmd_options
def downgrade(mongo_migrate, migration, fake=False):
    mongo_migrate.downgrade(migration, fake=fake)


@cli.command()
@mongo_migration_options
@click.argument("name", required=False)
def generate(mongo_migrate, name: Optional[str] = None):
    click.echo(f"Generating new migration in {mongo_migrate.migrations_dir}")
    file_path = mongo_migrate.generate(name)
    click.echo(
        "Generated: "
        + click.style(f"{file_path.parent}{os.sep}", fg="bright_black")
        + click.style(file_path.stem, fg="green")
        + click.style(file_path.suffix, fg="bright_black")
    )


@cli.command()
@mongo_migration_options
def graph(mongo_migrate):
    stdout = click.get_text_stream("stdout")
    dump(mongo_migrate.graph, stdout)


if __name__ == "__main__":
    cli()
